@@ -1,8 +1,7 @@
 import os
 import warnings
 from abc import abstractmethod
-from glob import glob
-from typing import Any, Dict, List, Optional, Union
+from typing import Any, Dict, List, Optional, Tuple, Union
 from uuid import uuid4
 
 import numpy as np
@@ -22,11 +21,10 @@ from dataquality.utils.hdf5_store import HDF5_STORE
 from dataquality.utils.thread_pool import ThreadPoolManager
 from dataquality.utils.vaex import (
     _join_in_out_frames,
-    _validate_unique_ids,
     concat_hdf5_files,
-    drop_empty_columns,
-    get_dup_ids,
-    valid_ids,
+    filter_df,
+    validate_ids_for_df,
+    validate_unique_ids,
 )
 
 DATA_FOLDERS = ["emb", "prob", "data"]
@@ -46,6 +44,10 @@ class BaseGalileoDataLogger(BaseGalileoLogger):
         self.meta: Dict[str, Any] = meta or {}
 
     def log(self) -> None:
+        """Writes input data to disk in .galileo/logs
+
+        If input data already exist, append new data to existing input file
+        """
         self.validate()
         write_input_dir = (
             f"{BaseGalileoLogger.LOG_FILE_DIR}/{config.current_project_id}/"
@@ -53,44 +55,53 @@ class BaseGalileoDataLogger(BaseGalileoLogger):
         )
         if not os.path.exists(write_input_dir):
             os.makedirs(write_input_dir)
+
         df = self._get_input_df()
         file_path = f"{write_input_dir}/{BaseGalileoDataLogger.INPUT_DATA_NAME}"
+
         if os.path.isfile(file_path):
-            new_name = f"{write_input_dir}/{str(uuid4()).replace('-', '')[:12]}.arrow"
-            os.rename(file_path, new_name)
-            logged_data = vaex.open(new_name)
-            combined_data = vaex.concat([logged_data, df])
-            # Validate there are no duplicated IDs
-            for split in df["split"].unique():
-                split_df = df[df["split"] == split]
-                if not valid_ids(split_df):
-                    dups = get_dup_ids(split_df)
-                    combined_data.close()
-                    os.rename(new_name, file_path)  # Revert name, we aren't logging
-                    raise GalileoException(
-                        "It seems the newly logged data has IDs that duplicate "
-                        f"previously logged data for split {split}. "
-                        f"Duplicated IDs: {dups}"
-                    )
-            with vaex.progress.tree("vaex", title="Appending input data"):
-                combined_data.export(file_path)
-            os.remove(new_name)
+            self.append_input_data(df, write_input_dir, file_path)
         else:
             with vaex.progress.tree("vaex", title="Exporting input data"):
                 df.export(file_path)
         df.close()
 
+    def append_input_data(
+        self, df: DataFrame, write_input_dir: str, file_path: str
+    ) -> None:
+        # Create a temporary file for existing input data
+        tmp_name = f"{write_input_dir}/{str(uuid4()).replace('-', '')[:12]}.arrow"
+        os.rename(file_path, tmp_name)
+
+        # Merge existing data with new data
+        existing_df = vaex.open(tmp_name)
+        merged_df = vaex.concat([existing_df, df])
+
+        try:  # Validate there are no duplicated IDs
+            validate_ids_for_df(df)
+        except GalileoException as e:  # Cleanup and raise on error
+            merged_df.close()
+            os.rename(tmp_name, file_path)  # Revert name, we aren't logging
+            raise e
+
+        with vaex.progress.tree("vaex", title="Appending input data"):
+            merged_df.export(file_path)
+
+        # Cleanup temporary file after appending input data
+        os.remove(tmp_name)
+
     @classmethod
     def upload(cls) -> None:
         """
-        Iterates through all of the splits/epochs/[data/emb/prob] folders, concatenates
-        all of the files with vaex, and uploads them to a single file in minio
+        Iterates through all of each splits children folders [data/emb/prob] for each
+        inference name / epoch, concatenates all of the files with vaex, and uploads
+        them to a single file in minio
         """
         ThreadPoolManager.wait_for_threads()
         print("☁️ Uploading Data")
+        object_store = ObjectStore()
         proj_run = f"{config.current_project_id}/{config.current_run_id}"
         location = f"{cls.LOG_FILE_DIR}/{proj_run}"
-        object_store = ObjectStore()
 
         in_frame = vaex.open(
             f"{location}/{BaseGalileoDataLogger.INPUT_DATA_NAME}"
@@ -100,55 +111,72 @@ class BaseGalileoDataLogger(BaseGalileoLogger):
             if not os.path.exists(split_loc):
                 continue
 
-            in_frame_split = in_frame[in_frame["split"].str.equals(split)].copy()
-            # Drop any columns for this split that are empty
-            # (ex: metadata logged for a different split)
-            in_frame_split = drop_empty_columns(in_frame_split)
-            # Remove the mask, work with only the filtered rows
-            in_frame_split = in_frame_split.extract()
-            for epoch_dir in glob(f"{split_loc}/*"):
-                epoch = int(epoch_dir.split("/")[-1])
-                # For all epochs that aren't the last 2 (early stopping), we only want
-                # to upload the probabilities (for DEP calculation).
-                if epoch < cls.logger_config.last_epoch - 1:
-                    prob_only = True
-                else:
-                    prob_only = False
+            in_frame_split = filter_df(in_frame, "split", split)
+            cls.upload_split(object_store, in_frame_split, split, split_loc)
 
-                str_cols = concat_hdf5_files(epoch_dir, prob_only)
-                out_frame = vaex.open(f"{epoch_dir}/{HDF5_STORE}")
-                # Post concat, string columns come back as bytes and need conversion
-                for col in str_cols:
-                    out_frame[col] = out_frame[col].to_arrow().cast(pa.large_string())
-                if prob_only:
-                    out_frame["epoch"] = vaex.vconstant(epoch, length=len(out_frame))
-                    out_frame["split"] = vaex.vconstant(
-                        split, length=len(out_frame), dtype="str"
-                    )
+    @classmethod
+    def upload_split(
+        cls,
+        object_store: ObjectStore,
+        in_frame: DataFrame,
+        split: str,
+        split_loc: str,
+    ) -> None:
+        split_runs = os.listdir(split_loc)
 
-                in_out_frames = cls.process_in_out_frames(
-                    in_frame_split, out_frame, prob_only
-                )
-                prob = in_out_frames.prob
-                emb = in_out_frames.emb
-                data_df = in_out_frames.data
+        for split_run in split_runs:  # For each inference name or epoch
+            in_frame_slice = in_frame.copy()
+            prob_only = cls.prob_only(split, split_run)
+            if split == Split.inference:
+                in_frame_slice = filter_df(in_frame_slice, "inference_name", split_run)
 
-                for data_folder, df_obj in tqdm(
-                    zip(DATA_FOLDERS, [emb, prob, data_df]), total=3, desc=split
-                ):
-                    if prob_only and data_folder != "prob":
-                        continue
-                    ext = cls.DATA_FOLDER_EXTENSION[data_folder]
-                    minio_file = (
-                        f"{proj_run}/{split}/{epoch}/{data_folder}/{data_folder}.{ext}"
-                    )
-                    object_store.create_project_run_object_from_df(
-                        df=df_obj, object_name=minio_file
-                    )
+            dir_name = f"{split_loc}/{split_run}"
+            in_out_frames = cls.create_in_out_frames(
+                in_frame_slice, dir_name, prob_only, split, split_run
+            )
+            cls.upload_in_out_frames(object_store, in_out_frames, split, split_run)
+
+    @classmethod
+    def create_in_out_frames(
+        cls,
+        in_frame: DataFrame,
+        dir_name: str,
+        prob_only: bool,
+        split: str,
+        split_run: Union[str, int],
+    ) -> DataFrame:
+        str_cols = concat_hdf5_files(dir_name, prob_only)
+        out_frame = vaex.open(f"{dir_name}/{HDF5_STORE}")
+
+        if split == Split.inference:
+            dtype: Union[str, None] = "str"
+            epoch_or_inf_name = "inference_name"
+        else:
+            dtype = None
+            epoch_or_inf_name = "epoch"
+
+        # Post concat, string columns come back as bytes and need conversion
+        for col in str_cols:
+            out_frame[col] = out_frame[col].to_arrow().cast(pa.large_string())
+        if prob_only:
+            out_frame["split"] = vaex.vconstant(
+                split, length=len(out_frame), dtype="str"
+            )
+            out_frame[epoch_or_inf_name] = vaex.vconstant(
+                split_run, length=len(out_frame), dtype=dtype
+            )
+
+        return cls.process_in_out_frames(
+            in_frame, out_frame, prob_only, epoch_or_inf_name
+        )
 
     @classmethod
     def process_in_out_frames(
-        cls, in_frame: DataFrame, out_frame: DataFrame, prob_only: bool
+        cls,
+        in_frame: DataFrame,
+        out_frame: DataFrame,
+        prob_only: bool,
+        epoch_or_inf_name: str,
     ) -> BaseLoggerInOutFrames:
         """Processes input and output dataframes from logging
 
@@ -156,12 +184,55 @@ class BaseGalileoDataLogger(BaseGalileoLogger):
         Joins inputs and outputs
         Splits the dataframes into prob, emb, and data for uploading to minio
         """
-        _validate_unique_ids(out_frame)
+        validate_unique_ids(out_frame, epoch_or_inf_name)
         in_out = _join_in_out_frames(in_frame, out_frame)
 
         prob, emb, data_df = cls.split_dataframe(in_out, prob_only)
+        # These df vars will be used in upload_in_out_frames
+        emb.set_variable("skip_upload", prob_only)
+        data_df.set_variable("skip_upload", prob_only)
+
         return BaseLoggerInOutFrames(prob=prob, emb=emb, data=data_df)
 
+    @classmethod
+    def upload_in_out_frames(
+        cls,
+        object_store: ObjectStore,
+        in_out_frames: BaseLoggerInOutFrames,
+        split: str,
+        split_run: Union[str, int],
+    ) -> None:
+        proj_run = f"{config.current_project_id}/{config.current_run_id}"
+
+        prob = in_out_frames.prob
+        emb = in_out_frames.emb
+        data_df = in_out_frames.data
+
+        for data_folder, df_obj in tqdm(
+            zip(DATA_FOLDERS, [emb, prob, data_df]), total=3, desc=split
+        ):
+            if df_obj.variables.get("skip_upload"):
+                continue
+
+            ext = cls.DATA_FOLDER_EXTENSION[data_folder]
+            minio_file = (
+                f"{proj_run}/{split}/{split_run}/" f"{data_folder}/{data_folder}.{ext}"
+            )
+            object_store.create_project_run_object_from_df(
+                df=df_obj, object_name=minio_file
+            )
+
+    @classmethod
+    def prob_only(cls, split: str, split_run: Union[int, str]) -> bool:
+        if split == Split.inference:
+            return False
+
+        # If split is not inference, split_run must be epoch
+        epoch = int(split_run)
+        # For all epochs that aren't the last 2 (early stopping), we only
+        # want to upload the probabilities (for DEP calculation).
+        return bool(epoch < cls.logger_config.last_epoch - 1)
+
     @classmethod
     @abstractmethod
     def validate_labels(cls) -> None:
@@ -237,7 +308,9 @@ class BaseGalileoDataLogger(BaseGalileoLogger):
 
     @classmethod
     @abstractmethod
-    def split_dataframe(cls, df: DataFrame, prob_only: bool) -> DataFrame:
+    def split_dataframe(
+        cls, df: DataFrame, prob_only: bool
+    ) -> Tuple[DataFrame, DataFrame, DataFrame]:
         ...
 
     @abstractmethod
